@@ -98,7 +98,17 @@ module Test
 
       def check_fail_exception(expected_message, options={}, &proc)
         normalizer = lambda do |actual_message|
-          actual_message.gsub(/^(?:[a-zA-Z]:|<internal:core> )?[^:\n]+:\d+:.+\n/,
+          # * X: is for Windows.
+          # * <internal:XXX> such as <internal:array> is for CRuby.
+          # * <internal:core> is for TruffleRuby.
+          actual_message.gsub(/^
+                               (?:
+                                 (?:[a-zA-Z]:)?[^:\n]+| # Normal path
+                                 <internal:.+?>| # CRuby's internal path
+                                 <internal:core>\s[^:\n]+ # TruffleRuby's internal path
+                               )?
+                               :\d+:.+\n # line and content
+                              /x,
                               "")
         end
         check_assertions(true,
@@ -306,6 +316,31 @@ EOM
             end
           end
 
+          def test_basic_object_inspection
+            basic_object_class = Class.new(BasicObject) do
+              [:respond_to?, :respond_to_missing?, :is_a?].each do |m|
+                define_method(m, ::Kernel.instance_method(m))
+              end
+
+              def inspect
+                "inspected"
+              end
+
+              def pretty_print(q)
+                q.text(inspect)
+              end
+            end
+            object1 = basic_object_class.new
+            object2 = basic_object_class.new
+            message = <<-EOM.chomp
+<inspected> expected but was
+<inspected>.
+EOM
+            check_fail(message) do
+              assert_equal(object1, object2)
+            end
+          end
+
           def test_multi_lines_result
             message = <<-EOM.chomp
 <#{AssertionMessage.convert("a\nb")}> expected but was
@@ -394,7 +429,7 @@ EOM
             ascii_8bit_string = utf8_string.dup.force_encoding("ascii-8bit")
             message = <<-EOM.chomp
 <#{utf8_string.inspect}>("UTF-8") expected but was
-<#{ascii_8bit_string.inspect}>("ASCII-8BIT").
+<#{ascii_8bit_string.inspect}>("#{Encoding::ASCII_8BIT.name}").
 EOM
             check_fail(message) do
               assert_equal(utf8_string, ascii_8bit_string)
@@ -1794,6 +1829,131 @@ message.
       end
     end
 
+    class TestAssertInDeltaBigDecimal < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_in_delta(BigDecimal("1.40000000000000000001"),
+                          BigDecimal("1.40000000000000000001"),
+                          0)
+        end
+      end
+
+      def test_pass_without_delta
+        check_nothing_fails do
+          assert_in_delta(BigDecimal("1.40000000000000000001"),
+                          BigDecimal("1.40000000000000000002"))
+        end
+      end
+
+      def test_pass_with_message
+        check_nothing_fails do
+          assert_in_delta(BigDecimal("1.40000000000000000001"),
+                          BigDecimal("1.50000000000000000001"),
+                          BigDecimal("0.1"),
+                          "message")
+        end
+      end
+
+      def test_pass_float_like_object
+        check_nothing_fails do
+          float_thing = Object.new
+          def float_thing.to_f
+            0.2
+          end
+          assert_in_delta(BigDecimal("0.1"),
+                          float_thing,
+                          BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_string_expected
+        check_nothing_fails do
+          assert_in_delta("0.5",
+                          BigDecimal("0.4"),
+                          BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_with_message
+        check_fail("message.\n" +
+                    "<0.50000000000000000001e0> -/+ <0.5e-1> was expected to include\n" +
+                    "<0.40000000000000000001e0>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.40000000000000000001e0> < " +
+                    "<0.50000000000000000001e0>-<0.5e-1>[0.45000000000000000001e0] <= " +
+                    "<0.50000000000000000001e0>+<0.5e-1>[0.55000000000000000001e0]" +
+                    ">") do
+          assert_in_delta(BigDecimal("0.50000000000000000001"),
+                          BigDecimal("0.40000000000000000001"),
+                          BigDecimal("0.05"),
+                          "message")
+        end
+      end
+
+      def test_fail_because_not_float_like_object
+        object = Object.new
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("The arguments must respond to to_f; " +
+                    "the first float did not.\n" +
+                    "<#{inspected_object}>.respond_to?(:to_f) expected\n" +
+                    "(Class: <Object>)") do
+          assert_in_delta(object,
+                          BigDecimal("0.4"),
+                          BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_with_float_like_object
+        object = Object.new
+        def object.to_f
+          0.4
+        end
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("<0.10000000000000000001e0> -/+ <0.1e0> was expected to include\n" +
+                    "<#{inspected_object}>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.10000000000000000001e0>-<0.1e0>[0.0] <= " +
+                    "<0.10000000000000000001e0>+<0.1e0>[0.2] < " +
+                    "<#{inspected_object}>" +
+                    ">") do
+          assert_in_delta(BigDecimal("0.10000000000000000001"),
+                          object,
+                          BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_because_negaitve_delta
+        check_fail("The delta should not be negative.\n" +
+                   "<-0.1e0> was expected to be\n>=\n<0.0>.") do
+          assert_in_delta(BigDecimal("0.5"),
+                          BigDecimal("0.4"),
+                          BigDecimal("-0.1"),
+                          "message")
+        end
+      end
+
+      def test_fail_without_delta
+        check_fail("<0.140200000000000000001e1> -/+ <0.001> was expected to include\n" +
+                    "<0.140400000000000000001e1>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.140200000000000000001e1>-<0.001>[0.140100000000000000001e1] <= " +
+                    "<0.140200000000000000001e1>+<0.001>[0.140300000000000000001e1] < " +
+                    "<0.140400000000000000001e1>" +
+                    ">") do
+          assert_in_delta(BigDecimal("1.40200000000000000001"),
+                          BigDecimal("1.40400000000000000001"))
+        end
+      end
+    end
+
     class TestAssertNotInDelta < Test::Unit::TestCase
       include AssertionCheckable
 
@@ -1893,6 +2053,149 @@ message.
       end
     end
 
+    class TestAssertNotInDeltaBigDecimal < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_not_in_delta(BigDecimal("1.42000000000000000001"),
+                              BigDecimal("1.42000000000000000002"),
+                              BigDecimal("0.000000000000000000005"))
+        end
+      end
+
+      def test_pass_without_delta
+        check_nothing_fails do
+          assert_not_in_delta(BigDecimal("1.40200000000000000001"),
+                              BigDecimal("1.40400000000000000001"))
+        end
+      end
+
+      def test_pass_with_message
+        check_nothing_fails do
+          assert_not_in_delta(BigDecimal("0.50000000000000000001"),
+                              BigDecimal("0.50000000000000000002"),
+                              BigDecimal("0.000000000000000000009"),
+                              "message")
+        end
+      end
+
+      def test_pass_float_like_object
+        check_nothing_fails do
+          float_thing = Object.new
+          def float_thing.to_f
+            0.2
+          end
+          assert_not_in_delta(BigDecimal("0.10000000000000000001"),
+                              float_thing,
+                              BigDecimal("0.09"))
+        end
+      end
+
+      def test_pass_string_epxected
+        check_nothing_fails do
+          assert_not_in_delta("0.5",
+                              BigDecimal("0.4"),
+                              BigDecimal("0.09"))
+        end
+      end
+
+      def test_fail
+        check_fail("<0.140000000000000000001e1> -/+ <0.5e-19> was expected to not include\n" +
+                    "<0.140000000000000000002e1>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.140000000000000000001e1>-<0.5e-19>[0.139999999999999999996e1] <= " +
+                    "<0.140000000000000000002e1> <= " +
+                    "<0.140000000000000000001e1>+<0.5e-19>[0.140000000000000000006e1]" +
+                    ">") do
+          assert_not_in_delta(BigDecimal("1.40000000000000000001"),
+                              BigDecimal("1.40000000000000000002"),
+                              BigDecimal("0.00000000000000000005"))
+        end
+      end
+
+      def test_fail_without_delta
+        check_fail("<0.140200000000000000001e1> -/+ <0.001> was expected to not include\n" +
+                    "<0.140210000000000000001e1>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.140200000000000000001e1>-<0.001>" +
+                    "[#{BigDecimal("1.40200000000000000001") - 0.001}] <= " +
+                    "<0.140210000000000000001e1> <= " +
+                    "<0.140200000000000000001e1>+<0.001>" +
+                    "[#{BigDecimal("1.40200000000000000001") + 0.001}]" +
+                    ">") do
+          assert_not_in_delta(BigDecimal("1.40200000000000000001"),
+                              BigDecimal("1.40210000000000000001"))
+        end
+      end
+
+      def test_fail_with_message
+        check_fail("message.\n" +
+                    "<0.50000000000000000001e0> -/+ <0.5e-19> was expected to not include\n" +
+                    "<0.50000000000000000002e0>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.50000000000000000001e0>-<0.5e-19>[0.49999999999999999996e0] <= " +
+                    "<0.50000000000000000002e0> <= " +
+                    "<0.50000000000000000001e0>+<0.5e-19>[0.50000000000000000006e0]" +
+                    ">") do
+          assert_not_in_delta(BigDecimal("0.50000000000000000001"),
+                              BigDecimal("0.50000000000000000002"),
+                              BigDecimal("0.00000000000000000005"),
+                              "message")
+        end
+      end
+
+      def test_fail_with_float_like_object
+        object = Object.new
+        def object.to_f
+          0.4
+        end
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("<0.1e0> -/+ <0.5e0> was expected to not include\n" +
+                    "<#{inspected_object}>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e0>-<0.5e0>[#{0.1 - 0.5}] <= " +
+                    "<#{inspected_object}> <= " +
+                    "<0.1e0>+<0.5e0>[#{0.1 + 0.5}]" +
+                    ">") do
+          assert_not_in_delta(BigDecimal("0.1"),
+                              object,
+                              BigDecimal("0.5"))
+        end
+      end
+
+      def test_fail_because_not_float_like_object
+        object = Object.new
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("The arguments must respond to to_f; " +
+                    "the first float did not.\n" +
+                    "<#{inspected_object}>.respond_to?(:to_f) expected\n" +
+                    "(Class: <Object>)") do
+          assert_not_in_delta(object,
+                              BigDecimal("0.4"),
+                              BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_because_negaitve_delta
+        check_fail("The delta should not be negative.\n" +
+                   "<-5.0e-20> was expected to be\n>=\n<0.0>.") do
+          assert_not_in_delta(BigDecimal("+0.500000000000000000001"),
+                              BigDecimal("+0.500000000000000000002"),
+                              BigDecimal("-0.00000000000000000005"),
+                              "message")
+        end
+      end
+    end
+
     class TestAssertInEpsilon < TestCase
       include AssertionCheckable
 
@@ -1992,6 +2295,155 @@ message.
       end
     end
 
+    class TestAssertInEpsilonBigDecimal < TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("1.0e1000"),
+                            BigDecimal("0.9e1000"),
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_without_epsilon
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("1.0e1000"),
+                            BigDecimal("0.9991e1000"))
+        end
+      end
+
+      def test_pass_with_message
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("1.0e1000"),
+                            BigDecimal("0.9e1000"),
+                            BigDecimal("0.1"),
+                            "message")
+        end
+      end
+
+      def test_pass_float_like_object
+        check_nothing_fails do
+          float_thing = Object.new
+          def float_thing.to_f
+            9000.0
+          end
+          assert_in_epsilon(BigDecimal("10000"),
+                            float_thing,
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_string_expected
+        check_nothing_fails do
+          assert_in_epsilon("10000",
+                            BigDecimal("9000"),
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_zero_expected
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("0"),
+                            BigDecimal("1e-1000"))
+        end
+      end
+
+      def test_pass_minus_expected
+        check_nothing_fails do
+          assert_in_epsilon(BigDecimal("-1"),
+                            BigDecimal("-1"))
+        end
+      end
+
+      def test_fail_with_message
+        check_fail("message.\n" +
+                    "<0.1e501> -/+ (<0.1e501> * <0.1e0>)[0.1e500] " +
+                    "was expected to include\n" +
+                    "<0.8999e500>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.8999e500> < " +
+                    "<0.1e501>-(<0.1e501>*<0.1e0>)[0.9e500] <= " +
+                    "<0.1e501>+(<0.1e501>*<0.1e0>)[0.11e501]" +
+                    ">") do
+          assert_in_epsilon(BigDecimal("1e500"),
+                            BigDecimal("0.8999e500"),
+                            BigDecimal("0.1"),
+                            "message")
+        end
+      end
+
+      def test_fail_because_not_float_like_object
+        object = Object.new
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("The arguments must respond to to_f; " +
+                    "the first float did not.\n" +
+                    "<#{inspected_object}>.respond_to?(:to_f) expected\n" +
+                    "(Class: <Object>)") do
+          assert_in_epsilon(object,
+                            BigDecimal("0.9e500"),
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_with_float_like_object
+        if RUBY_ENGINE == "truffleruby"
+          omit("This fails on TruffleRuby; " +
+               "see https://github.com/test-unit/test-unit/pull/218 for details")
+        end
+
+        object = Object.new
+        def object.to_f
+          0.4
+        end
+        check_fail("<0.10000000000000000001e0> -/+ (<0.10000000000000000001e0> * <0.1e0>)" +
+                   "[0.010000000000000002] was expected to include\n" +
+                   "<#{object}>.\n" +
+                   "\n" +
+                   "Relation:\n" +
+                   "<" +
+                   "<0.10000000000000000001e0>-(<0.10000000000000000001e0>*<0.1e0>)" +
+                   "[0.09] <= " +
+                   "<0.10000000000000000001e0>+(<0.10000000000000000001e0>*<0.1e0>)" +
+                   "[0.11000000000000001] < " +
+                   "<#{object}>" +
+                   ">"
+                  ) do
+          assert_in_epsilon(BigDecimal("0.10000000000000000001"),
+                            object,
+                            BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_because_negaitve_epsilon
+        check_fail("The epsilon should not be negative.\n" +
+                    "<-0.1e0> was expected to be\n>=\n<0.0>.") do
+          assert_in_epsilon(BigDecimal("1.0e500"),
+                            BigDecimal("0.9e500"),
+                            BigDecimal("-0.1"),
+                            "message")
+        end
+      end
+
+      def test_fail_without_epsilon
+        check_fail("<0.1e501> -/+ (<0.1e501> * <0.001>)[0.1e498] " +
+                    "was expected to include\n" +
+                    "<0.10011e501>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e501>-(<0.1e501>*<0.001>)[0.999e500] <= " +
+                    "<0.1e501>+(<0.1e501>*<0.001>)[0.1001e501] < " +
+                    "<0.10011e501>" +
+                    ">") do
+          assert_in_epsilon(BigDecimal("1.0000e500"),
+                            BigDecimal("1.0011e500"))
+        end
+      end
+    end
+
     class TestAssertNotInEpsilon < Test::Unit::TestCase
       include AssertionCheckable
 
@@ -2075,6 +2527,28 @@ message.
         end
       end
 
+      def test_fail_with_float_like_object
+        object = Object.new
+        def object.to_f
+          0.4
+        end
+        check_fail("<0.1e1001> -/+ (<0.1e1001> * <0.1e0>)" +
+                   "[Infinity] was expected to not include\n" +
+                   "<#{object}>.\n" +
+                   "\n" +
+                   "Relation:\n" +
+                   "<" +
+                   "<0.1e1001>-(<0.1e1001>*<0.1e0>)[NaN] <= " +
+                   "<#{object}> <= " +
+                   "<0.1e1001>+(<0.1e1001>*<0.1e0>)[Infinity]" +
+                   ">"
+                  ) do
+          assert_not_in_epsilon(BigDecimal("1e1000"),
+                                object,
+                                BigDecimal("0.1"))
+        end
+      end
+
       def test_fail_because_not_float_like_object
         object = Object.new
         inspected_object = AssertionMessage.convert(object)
@@ -2094,6 +2568,129 @@ message.
       end
     end
 
+    class TestAssertNotInEpsilonBigDecimal < Test::Unit::TestCase
+      include AssertionCheckable
+
+      def test_pass
+        check_nothing_fails do
+          assert_not_in_epsilon(BigDecimal("1.0000e1000"),
+                                BigDecimal("0.8999e1000"),
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_without_epsilon
+        check_nothing_fails do
+          assert_not_in_epsilon(BigDecimal("1.0000e1000"),
+                                BigDecimal("0.9989e1000"))
+        end
+      end
+
+      def test_pass_with_message
+        check_nothing_fails do
+          assert_not_in_epsilon(BigDecimal("1.0000e1000"),
+                                BigDecimal("0.8999e1000"),
+                                BigDecimal("0.1"),
+                                "message")
+        end
+      end
+
+      def test_pass_float_like_object
+        check_nothing_fails do
+          float_thing = Object.new
+          def float_thing.to_f
+            8999.0
+          end
+          assert_not_in_epsilon(BigDecimal("10000"),
+                                float_thing,
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_pass_string_epxected
+        check_nothing_fails do
+          assert_not_in_epsilon("10000",
+                                BigDecimal("8999"),
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail
+        check_fail("<0.1e1001> -/+ (<0.1e1001> * <0.1e0>)[0.1e1000] " +
+                    "was expected to not include\n" +
+                    "<0.9e1000>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e1001>-(<0.1e1001>*<0.1e0>)[0.9e1000] <= " +
+                    "<0.9e1000> <= " +
+                    "<0.1e1001>+(<0.1e1001>*<0.1e0>)[0.11e1001]" +
+                    ">") do
+          assert_not_in_epsilon(BigDecimal("1.0e1000"),
+                                BigDecimal("0.9e1000"),
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_without_epsilon
+        check_fail("<0.1e1001> -/+ (<0.1e1001> * <0.001>)[0.1e998] " +
+                    "was expected to not include\n" +
+                    "<0.999e1000>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e1001>-(<0.1e1001>*<0.001>)[0.999e1000] <= " +
+                    "<0.999e1000> <= " +
+                    "<0.1e1001>+(<0.1e1001>*<0.001>)[0.1001e1001]" +
+                    ">") do
+          assert_not_in_epsilon(BigDecimal("1.000e1000"),
+                                BigDecimal("0.999e1000"))
+        end
+      end
+
+      def test_fail_with_message
+        check_fail("message.\n" +
+                    "<0.1e1001> -/+ (<0.1e1001> * <0.1e0>)[0.1e1000] " +
+                    "was expected to not include\n" +
+                    "<0.9e1000>.\n" +
+                    "\n" +
+                    "Relation:\n" +
+                    "<" +
+                    "<0.1e1001>-(<0.1e1001>*<0.1e0>)[0.9e1000] <= " +
+                    "<0.9e1000> <= " +
+                    "<0.1e1001>+(<0.1e1001>*<0.1e0>)[0.11e1001]" +
+                    ">") do
+          assert_not_in_epsilon(BigDecimal("1.0e1000"),
+                                BigDecimal("0.9e1000"),
+                                BigDecimal("0.1"),
+                                "message")
+        end
+      end
+
+      def test_fail_because_not_float_like_object
+        object = Object.new
+        inspected_object = AssertionMessage.convert(object)
+        check_fail("The arguments must respond to to_f; " +
+                    "the first float did not.\n" +
+                    "<#{inspected_object}>.respond_to?(:to_f) expected\n" +
+                    "(Class: <Object>)") do
+          assert_not_in_epsilon(object,
+                                BigDecimal("0.9e1000"),
+                                BigDecimal("0.1"))
+        end
+      end
+
+      def test_fail_because_negaitve_epsilon
+        check_fail("The epsilon should not be negative.\n" +
+                    "<-0.1e0> was expected to be\n>=\n<0.0>.") do
+          assert_not_in_epsilon(BigDecimal("10000"),
+                                BigDecimal("9000"),
+                                BigDecimal("-0.1"),
+                                "message")
+        end
+      end
+    end
+
     class TestAssertInclude < Test::Unit::TestCase
       include AssertionCheckable
 
@@ -2312,6 +2909,12 @@ EOM
     class TestAssertNothingLeakedMemory < Test::Unit::TestCase
       include AssertionCheckable
 
+      class << self
+        def parallel_safe?
+          false
+        end
+      end
+
       def setup
         @data = "Hello!" * 100
         if ObjectSpace.respond_to?(:memsize_of)
@@ -2361,6 +2964,10 @@ EOM
       end
 
       def test_fail
+        if RUBY_ENGINE == "truffleruby"
+          omit("This fails on TruffleRuby; String#dup may not copy data.")
+        end
+
         actual_increased_size = 10000
         n_tries = 100_000
         max_increasable_size = -1
